#include "potion.h"

#include "constants.h"

potion::potion(const potion_type type, const int duration, const position &pos):
        type{type}, remaining_duration{duration}, pos{pos} {}

potion_type potion::get_type() const {
        return type;
}

int potion::get_duration() const {
        return remaining_duration;
}

position potion::get_pos() const {
        return pos;
}

void potion::set_pos(const position &npos) {
        pos = npos;
}

potion::potion(const potion &p):
        type(p.type), remaining_duration(p.remaining_duration),
        pos{p.pos} {}

potion::potion(potion &&p):
        type(p.type), remaining_duration(p.remaining_duration),
        pos{p.pos} {}

potion &potion::operator=(const potion &p) {
        type = p.type;
        remaining_duration = p.remaining_duration;
        pos = p.pos;
        return *this;
}

potion &potion::operator=(potion &&p) {
        type = p.type;
        remaining_duration = p.remaining_duration;
        pos = p.pos;
        return *this;
}

void potion::print(output *out) {
        out->print_char(pos, 'P', COLOR_PAIR(COLOR_GREEN));
}

potion *get_potion_at(const position &pos, potion_list &plist) {
        potion *ret = nullptr;

        for (size_t i = 0; i < plist.size(); ++i)
                if (plist[i]->get_pos() == pos) {
                        ret = plist[i];
                        plist.erase(plist.begin() + i);
                        return ret;
                }

        return ret;
}
